package be.iminds.ilabt.jfed.experiment.setup;

import be.iminds.ilabt.jfed.experiment.setup.config.*;
import com.fasterxml.jackson.annotation.*;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.dropwizard.jackson.Jackson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Objects;

public class ExperimentSetupConfig {
    private static final Logger LOG = LoggerFactory.getLogger(ExperimentSetupConfig.class);
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

    @Nonnull protected final Slice slice;
    @Nullable protected final RequestRSpec requestRSpec;
    @Nullable protected final ESpec eSpec;
    @Nonnull protected final Provision provision;
    @Nonnull protected final WaitForReady waitForReady;
    protected final boolean overrideESpecRSpec;
    protected final boolean runLinkTest;

    public ExperimentSetupConfig() {
        this.slice = new Slice();
        this.requestRSpec = RequestRSpec.getDefault();
        this.eSpec = null;
        this.provision = new Provision();
        this.waitForReady = new WaitForReady();
        this.overrideESpecRSpec = false;
        this.runLinkTest = false;
    }

    @JsonCreator
    public ExperimentSetupConfig(@Nullable @JsonProperty("slice") Slice slice,
                    @Nullable @JsonProperty("requestRSpec") RequestRSpec requestRSpec,
                    @Nullable @JsonProperty("eSpec") ESpec eSpec,
                    @Nullable @JsonProperty("provision") Provision provision,
                    @Nullable @JsonProperty("waitForReady") WaitForReady waitForReady,
                    @Nullable @JsonProperty("overrideESpecRSpec") Boolean overrideESpecRSpec,
                    @Nullable @JsonProperty("runLinkTest") Boolean runLinkTest) {
        this.slice = slice == null ? new ExperimentSetupConfig().getSlice() : slice;
        this.requestRSpec = requestRSpec == null ?
                (eSpec == null ? new ExperimentSetupConfig().getRequestRSpec() : null) :
                requestRSpec;
        this.eSpec = eSpec == null ? null : eSpec;
        this.provision = provision == null ? new ExperimentSetupConfig().getProvision() : provision;
        this.waitForReady = waitForReady == null ? new ExperimentSetupConfig().getWaitForReady() : waitForReady;
        this.overrideESpecRSpec = overrideESpecRSpec == null ? new ExperimentSetupConfig().getOverrideESpecRSpec() : overrideESpecRSpec;
        this.runLinkTest = runLinkTest == null ? new ExperimentSetupConfig().getRunLinkTest() : runLinkTest;
    }

    @JsonIgnore
    public boolean isValid() {
        boolean res = getSlice().isValid();
        RequestRSpec requestRspec = getRequestRSpec();
        ESpec espec = getESpec();
        res = res && (requestRspec == null || requestRspec.isValid());
        res = res && (espec == null || espec.isValid());
        res = res && (espec != null || requestRspec != null);
        res = res && getProvision().isValid();
        res = res && getWaitForReady().isValid();
        if (!getProvision().isEnabled() && !getWaitForReady().isEnabled())
            throw new IllegalArgumentException("Config Error: if the Provision step is disabled, waitForReady should be disabled too.");
        if (getOverrideESpecRSpec()) {
            if (espec == null || requestRspec == null)
                throw new IllegalArgumentException("Config Error: overrideESpecRSpec=true requires specifying both an RSpec and an ESpec");
        } else {
            if (espec != null && requestRspec != null)
                throw new IllegalArgumentException("Config Error: Specifying both an RSpec and an ESpec requires overrideESpecRSpec=true");
        }
        return res;
    }

    @JsonProperty
    @Nonnull
    public Slice getSlice() {
        return slice;
    }

    @JsonProperty
    @Nullable
    public RequestRSpec getRequestRSpec() {
        return requestRSpec;
    }

    @JsonProperty("eSpec")
    @Nullable
    public ESpec getESpec() {
        return eSpec;
    }

    @JsonProperty
    @Nonnull
    public Provision getProvision() {
        return provision;
    }

    @JsonProperty
    @Nonnull
    public WaitForReady getWaitForReady() {
        return waitForReady;
    }


    @JsonProperty("overrideESpecRSpec")
    public boolean getOverrideESpecRSpec() {
        return overrideESpecRSpec;
    }

    @JsonProperty("runLinkTest")
    public boolean getRunLinkTest() {
        return runLinkTest;
    }


    ;


    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof ExperimentSetupConfig)) return false;
        ExperimentSetupConfig that = (ExperimentSetupConfig) o;
        return overrideESpecRSpec == that.overrideESpecRSpec &&
                runLinkTest == that.runLinkTest &&
                Objects.equals(slice, that.slice) &&
                Objects.equals(requestRSpec, that.requestRSpec) &&
                Objects.equals(eSpec, that.eSpec) &&
                Objects.equals(provision, that.provision) &&
                Objects.equals(waitForReady, that.waitForReady);
    }

    @Override
    public int hashCode() {
        return Objects.hash(slice, requestRSpec, eSpec, provision, waitForReady, overrideESpecRSpec, runLinkTest);
    }
}
